import QtQuick 1.1
import com.nokia.meego 1.0
import "uiconstants.js" as UiContants

//MainPage.qml: Main UI Window / display of user playlists and search / settings related elements.

Page {

    id: mainPage

    property int currPlaylist: -1

    /* toolbar element for the page */
    tools: commonToolBarLayout


    /* main menu selection dialog */
    SelectionDialog    {
        id: mainMenuDialog
        titleText: qsTr("Main Menu")
        model: ListModel{
            id: mainMenuModel
            ListElement { name: "Settings" }
            ListElement { name: "About" }
            ListElement { name: "Log Out" }
        }

        onSelectedIndexChanged:  {
            if(mainMenuDialog.selectedIndex===0)
            {
                //settingsSheet.open(); //show the settings page / sheet
                settingsSheet.open();
            }
            else if(mainMenuDialog.selectedIndex===1)
            {
                aboutDialog.setOfflineTimeLeftFormatted(qspotCore.offlineTimeLeft()); //set current offline time left (if available)
                aboutDialog.open(); //show the app about dialog
            }
            else if(mainMenuDialog.selectedIndex===2) //Log out / forget credentials for the current user
            {
               qspotCore.forgetUser();
               qspotCore.logout();
            }
        }
    }

    PageHeader {
        id: playlistPageHeader
        text: qsTr("My Playlists")
        anchors.left: parent.left
        anchors.top: parent.top

        onClicked: { //display the main menu selection dialog
            mainMenuDialog.selectedIndex = -1;
            mainMenuDialog.open();
        }
    }


    //Search input field
    TextField {
        id: searchInput
        inputMethodHints: Qt.ImhNoAutoUppercase | Qt.ImhNoPredictiveText
        font.family: UiContants.DEFAULT_FONT_FAMILY
        font.pointSize: UiContants.DEFAULT_TEXT_POINTSIZE
        anchors.left: parent.left
        anchors.top: playlistPageHeader.bottom
        anchors.topMargin: 10
        anchors.rightMargin: 5
        anchors.right: parent.right
        anchors.bottomMargin: 5
        placeholderText: qsTr("Search")
        platformStyle: TextFieldStyle { paddingRight: clearButton.width }

        Image { //clear input button
            id: clearButton
            anchors.right: parent.right
            anchors.verticalCenter: parent.verticalCenter
            source: "image://theme/icon-m-input-clear"

            MouseArea {
                anchors.fill: parent

                onClicked: { //clear the search field content
                    inputContext.reset();
                    searchInput.text = "";
                }

            }
        }

        Keys.onReturnPressed: {
            if(searchInput.text!=="")
            {//do search
                playlistView.focus = true;
                searchInput.enabled = false;
                busyIndicator.text = qsTr("Searching");
                busyIndicator.running = true;
                busyIndicator.visible = true;
                if(!qspotCore.doSearch(searchInput.text)) //perform search
                {
                    //handle error
                    inputContext.reset();
                    searchInput.text = "";
                    busyIndicator.running = false;
                    busyIndicator.visible = false;
                    searchInput.enabled = true;
                }
            }
        }
    }

    Separator {
        id: viewSeparator
        anchors.top: searchInput.bottom
        anchors.topMargin: 5
        width: parent.width
    }

    ProgressBar {
        id: syncProgressIndicator
        minimumValue: 0
        maximumValue: 100
        value: 0
        visible: false
        anchors.left: parent.left
        anchors.leftMargin: 5
        anchors.right: parent.right
        anchors.rightMargin: 5
        anchors.top: viewSeparator.bottom
        anchors.topMargin: 5
    }


    //ListView element for holding user playlists (playlistcontainer)

    ListView
    {
        id: playlistView
        anchors.top: syncProgressIndicator.bottom
        anchors.topMargin: 5
        anchors.bottom: parent.bottom
        anchors.left: parent.left
        anchors.leftMargin: 5
        anchors.right: parent.right
        anchors.rightMargin: 5
        snapMode: ListView.SnapToItem
        model:  qspotCore.playlistModel
        clip: true
        interactive: true
        delegate:   PlaylistInfoDelegate
        {
           id: playlistItemDelegate
           onClicked: {
                    playlistView.currentIndex = index; //get the index of playlist item clicked (activated)
                    qspotCore.setActivePlaylist(position); //set active playlist based on user's selection (position in internal list)
                    playlistPage.playlistName = name; //assign playlist name as page header
                    playlistPage.isTracksOffline = isOffline; //set whether playlist is available offline
                    pageStack.push(playlistPage); //push playlist tracks page to stack (i.e., display it)
                }
           onPressAndHold: { /* display pop-up playlist options menu */
               playlistView.currentIndex = index;
               currPlaylist = position; //set position of pressed playlist (from model)
               if(!isOffline) //playlist not available offline
                    playlistOfflineStateToggle.text = qsTr("Set available offline");
               else //playlist available offline
                   playlistOfflineStateToggle.text = qsTr("Unset available offline");
               playlistOptions.open(); //display menu
           }
          }
      }

    ScrollDecorator {
        flickableItem: playlistView
    }

//signal / slot connections
Connections
{
    target: qspotCore

    onLoginResponse:
    {
        if(status===false) //no error flag set
        { //successful login against Spotify backend

            busyIndicator.text = qsTr("Loading playlists");
        }
        else
        {
           //display error notification on failed login
            busyIndicator.running = false;
            busyIndicator.visible = false;
            messageBox.dialogTitle = qsTr("Some problem occured");
            messageBox.textContent = errorDesc;
            messageBox.open();
        }

    }

    onLoggedOutResponse: { //user has been logged out from Spotify backed
        Qt.quit();
    }

    onPlaylistsReady: { //user's playlists successfully loaded
        //stop / hide busy indicator
        busyIndicator.running = false;
        busyIndicator.visible = false;
        searchInput.enabled = !qspotCore.offlineMode; //enable the search field in case offline mode is not active
        qspotCore.initAudioBackend(); //init pulseaudio interface
    }

    onSearchComplete: { //search operation has completed
        busyIndicator.running = false;
        busyIndicator.visible = false;
        searchResultsPage.resetPageState();
        if(status===true)
        {
            if(numResults>0)
            {
                pageStack.push(searchResultsPage); //display search results page
            }
            else
            { //no results returned; display message to user
               info.text = qsTr("Search returned no results!");
               info.show();
            }
        }
        else
        { //some error occured; notify the user
            info.text = qsTr("Search failed: ") + " " + errDesc;
            info.show();
        }
        inputContext.reset();
        searchInput.text = "";
        searchInput.enabled = true;
    }

    onOfflineStateChanged: { //offline param is true in case offline mode is active
        searchInput.text = "";
        searchInput.enabled = !offline;
        if(offline)
        { //notify user about state change to offline mode
            info.text = qsTr("Offline mode is now actived");
            info.show();
        }
    }

    onOfflineSyncStatusUpdated: { //offline synchronisation status updated
        if(newSync)
        { //new offline sync. operation initiated
            info.text = qsTr("Offline synchronisation of playlists in progress");
            info.show();
            if(!syncProgressIndicator.visible)
                syncProgressIndicator.visible = true;
        }
        syncProgressIndicator.value = progress; //update progress
        if(progress===syncProgressIndicator.maximumValue)
            syncProgressIndicator.visible = false; //hide progress bar once sync. operation is completed
    }

    onOfflineSyncError: { //notify user about error with offline synchronisation
        info.text = qsTr("Error") + ": " + errDesc;
        info.show();
    }

    onPlaybackTokenLost: { //playback token has been lost; notify user
        info.text = qsTr("Playback paused due to the same account being used somewhere else!");
        info.show();
    }

    onPlaybackStopped: { //playback stopped (by system) due to i.e, switch to offline mode
        if(queuePage.status===2||(playlistPage.status===2 && !playlistPage.isTracksOffline)||searchResultsPage.status===2)
        { //active page is Now Playing (queue); force view to main (playlists) page
            pageStack.clear();
            pageStack.push(mainPage);
        }
        queuePage.resetQueue();
    }

    onAuthenticateSessionStatus: { /* authentication status received from Last.FM service */

        if(status)
        {
            info.text = qsTr("Successfully authenticated with Last.FM");
        }
        else
        {
            info.text = qsTr("Error occured during Last.FM authentication: ") + errDesc;
        }
        info.show();
    }
}

/* menu providing options for selected (long-pressed) playlist */
Menu
{
    id: playlistOptions
    content: MenuLayout {
        MenuItem {
            id: playlistOfflineStateToggle
            onClicked: {
                if(currPlaylist!==-1)
                    qspotCore.togglePlaylistOfflineState(currPlaylist);//set / unset offline state for selected playlist
            }
        }
    }
}

Component.onCompleted:
{ //perform initialisation
    searchInput.enabled = false;
}

/* Settings sheet */

Sheet
{
    id: settingsSheet

    property int numTracksValue: -1
    property int numAlbumsValue: -1
    property int numArtistsValue: -1
    property int minBufferLenValue: -1

    /* save current (changed) settings to config file */
    function saveSettings()
    {
        numTracksValue = numTracksReturned.text*1;
        numAlbumsValue = numAlbumsReturned.text*1;
        numArtistsValue = numArtistsReturned.text*1;
        minBufferLenValue = minBufferLen.text*1;

        if(scrobbleToLastFM.checked)
        { //authenticate with Last.FM service
            qspotCore.setLastFMScrobbling(true);
            qspotCore.authLastFM(lastFMUserID.text,lastFMPassword.text);
        }
        if(numTracksValue!==-1)
            qspotCore.setMaxTracksFromSearch(numTracksValue);
         if(numAlbumsValue!==-1)
            qspotCore.setMaxAlbumsFromSearch(numAlbumsValue);
        if(numArtistsValue!==-1)
            qspotCore.setMaxArtistsFromSearch(numArtistsValue);
        if(minBufferLenValue!==-1)
            qspotCore.setMinBufferLength(minBufferLenValue);
    }

    acceptButtonText: qsTr("Done")

    title: Label { anchors.centerIn: parent; text: qsTr("Settings") }

    content: Flickable {

        id: mainContent
        anchors.fill: parent
        anchors.leftMargin: 16
        anchors.topMargin: 16
        anchors.rightMargin: 16
        contentWidth: mainColumn.width
        contentHeight: mainColumn.height
        clip: true
        flickableDirection: Flickable.VerticalFlick

        ListModel {
            id: bitrateModel
            ListElement { name: "Low bitrate (96k)" }
            ListElement { name: "Medium bitrate (160k)"}
            ListElement { name: "High bitrate (320k)" }
        }

        Column {

            id: mainColumn
            anchors.top: parent.top
            spacing: 20


            Row {

                anchors.left: parent.left
                anchors.right: parent.right
                spacing: 20

                Switch {
                    id: forceOfflineMode
                    checked: qspotCore.offlineMode
                    platformStyle: SwitchStyle {}
                    onCheckedChanged: {
                        qspotCore.setOfflineMode(checked);
                    }
                }

                Label {
                    text: qsTr("Force offline mode")
                    height: forceOfflineMode.height
                    verticalAlignment: Text.AlignVCenter
                }
            }

            Separator {
                anchors.left: parent.left
                width: mainContent.width
            }

            Row {

                anchors.left: parent.left
                anchors.right: parent.right
                spacing: 20

                Switch {
                    id: allowMobileSync
                    checked: qspotCore.allowMobileSync
                    platformStyle: SwitchStyle {}
                    onCheckedChanged: {
                        qspotCore.setAllowMobileSync(checked);
                    }
                }

                Label {
                    text: qsTr("Allow sync over 2G/3G")
                    height: allowMobileSync.height
                    verticalAlignment: Text.AlignVCenter
                }
            }

            Separator {
                anchors.left: parent.left
                width: mainContent.width
            }

            SelectionListItem {
                id: streamBitrateSelItem
                title: qsTr("Streaming bitrate")
                selectedIndex: qspotCore.streamingBitrate
                model: bitrateModel
                onItemSelectionChanged: {
                    qspotCore.setStreamingBitrate(streamBitrateSelItem.selectedIndex);
                }
            }

            SelectionListItem {
                id: offlineSyncBitrateSelItem
                title: qsTr("Offline sync bitrate")
                selectedIndex: qspotCore.offlineSyncBitrate
                model: bitrateModel
                onItemSelectionChanged: {
                    qspotCore.setOfflineSyncBitrate(offlineSyncBitrateSelItem.selectedIndex);
                }
            }

            Separator {
                anchors.left: parent.left
                width: mainContent.width
            }

            Row {
                anchors.left: parent.left
                anchors.right: parent.right
                spacing: 20

                Label { text: qsTr("Min. buffer length (secs)");  anchors.verticalCenter: parent.verticalCenter }
                TextField { id: minBufferLen; text: qspotCore.minBufferLength; inputMask: "000";  anchors.verticalCenter: parent.verticalCenter;  width: 100}
            }

            Separator {
                anchors.left: parent.left
                width: mainContent.width
            }

            Row {

                anchors.left: parent.left
                anchors.right: parent.right
                spacing: 20

                Switch {
                    id: volumeNormalisation
                    checked: qspotCore.volumeNormalisation
                    platformStyle: SwitchStyle {}
                    onCheckedChanged: {
                        qspotCore.setVolumeNormalisation(checked);
                    }
                }

                Label {
                    text: qsTr("Enable volume normalisation")
                    height: volumeNormalisation.height
                    verticalAlignment: Text.AlignVCenter
                }
            }

            Separator {
                anchors.left: parent.left
                width: mainContent.width
            }

            Row {

                anchors.left: parent.left
                anchors.right: parent.right
                spacing: 20

                Switch {
                    id: scrobbleToLastFM
                    checked: qspotCore.lastFMScrobbling
                    platformStyle: SwitchStyle {}
                    onCheckedChanged: {
                        if(!checked)
                        {
                            qspotCore.setLastFMScrobbling(false);
                            lastFMUserID.text = "";
                            lastFMPassword.text = "";
                        }
                    }
                }

                Label {
                    text: qsTr("Scrobble to Last.FM")
                    height: scrobbleToLastFM.height
                    verticalAlignment: Text.AlignVCenter
                }
            }

            Row {
                anchors.left: parent.left
                anchors.right: parent.right
                spacing: 20
                visible: scrobbleToLastFM.checked

                Label { text: qsTr("Last.FM username");  anchors.verticalCenter: parent.verticalCenter }
                TextField { id: lastFMUserID; text: qspotCore.lastFMUserID; anchors.verticalCenter: parent.verticalCenter; width: 200 }
            }

            Row {
                anchors.left: parent.left
                anchors.right: parent.right
                spacing: 20
                visible: scrobbleToLastFM.checked

                Label { text: qsTr("Last.FM password");  anchors.verticalCenter: parent.verticalCenter }
                TextField { id: lastFMPassword; text: qspotCore.lastFMPassword; anchors.verticalCenter: parent.verticalCenter; width: 200; echoMode: TextInput.Password}
            }

            Separator {
                anchors.left: parent.left
                width: mainContent.width
            }

            Row {

                anchors.left: parent.left
                anchors.right: parent.right
                spacing: 20

                Switch {
                    id: enableEventFeed
                    checked: qspotCore.enableEventFeed
                    platformStyle: SwitchStyle {}
                    onCheckedChanged: {
                        qspotCore.setEventFeedState(checked);
                    }
                }

                Label {
                    text: qsTr("Show Now Playing in event view")
                    height: enableEventFeed.height
                    verticalAlignment: Text.AlignVCenter
                }
            }

            Separator {
                anchors.left: parent.left
                width: mainContent.width
            }

            Label { text: qsTr("Search options") }

            Row {
                anchors.left: parent.left
                anchors.right: parent.right
                spacing: 20
                Label { text: qsTr("Max tracks returned"); anchors.verticalCenter: parent.verticalCenter }
                TextField { id: numTracksReturned; text: qspotCore.maxTracksFromSearch; inputMask: "0000"; anchors.verticalCenter: parent.verticalCenter; width: 100 }
            }

            Row {
                anchors.left: parent.left
                anchors.right: parent.right
                spacing: 20
                Label { text: qsTr("Max albums returned"); anchors.verticalCenter: parent.verticalCenter }
                TextField { id: numAlbumsReturned; text: qspotCore.maxAlbumsFromSearch; inputMask: "0000"; anchors.verticalCenter: parent.verticalCenter; width: 100 }
            }

            Row {
                anchors.left: parent.left
                anchors.right: parent.right
                spacing: 20

                Label { text: qsTr("Max artists returned"); anchors.verticalCenter: parent.verticalCenter  }
                TextField { id: numArtistsReturned; text: qspotCore.maxArtistsFromSearch; inputMask: "0000"; anchors.verticalCenter: parent.verticalCenter; width: 100  }
            }

        }
    }

    onAccepted: {  //save current settings
        saveSettings(); }
}

}





