import QtQuick 1.1
import com.nokia.meego 1.0
import "uiconstants.js" as UiConstants

/* Implementation of the Queue (Now Playing) page, providing information about the currently playing track (and other queued tracks). */

Page
{
    id: queuePage

    orientationLock: PageOrientation.LockPortrait /* locked until landscape properly implemented */

    /* misc. properties */
    property int currState: UiConstants.QUEUEPAGE_STATE_NORMAL //default state
    property int currTrackDuration: 0 /* duration of currently active (playing) track in secs. Used for e.g. calculating playback progress to be displayed.*/
    property int currPlaybackPos: 0 /* current playback position */
    property int currBufferingPos: 0 /* current buffering position */
    property bool isPlaying: false  /* true in case playback is active; false otherwise */

    /* misc. helper methods */

    function resetQueue()
    {//set the queue to not active state
        currTrackDuration = 0;
        albumCover.source = "";
    }

    function centerViewAtCurrTrack()
    { //center listview so that currently playing track is visible
        playQueueView.positionViewAtIndex(playQueueView.currentIndex,ListView.Center);
    }

    function setActiveTrack(index,centerItem)
    {
        //set active track in the current playqueue (e.g., start playback). Index refers to position in playqueue (listview)
        var defaultDuration = playQueueView.highlightMoveDuration;
        playQueueView.highlightMoveDuration = 1;
        playQueueView.currentIndex = index;
        playQueueView.highlightMoveDuration = defaultDuration;
        if(centerItem===true)
            playQueueView.positionViewAtIndex(index,ListView.Center);
        qspotCore.playQueuedTrack(index); //request to play selected track
        isPlaying = true;
    }

    function setCurrentTrackDetails(trackDurationSecs, trackDurationFormatted, trackName, trackArtist)
    {
        queuePage.currTrackDuration = trackDurationSecs;
        progressIndicator.indicatorLabelLeft = "0:00";
        progressIndicator.indicatorLabelRight = trackDurationFormatted; //formatted duration value
        progressIndicator.indicator1Pos = 0; //reset playback indicator
        progressIndicator.indicator2Pos = 0; //reset buffering indicator
        trackNameLabel.text = trackName; //assign track name
        artistNameLabel.text = trackArtist; //assign artist name
    }

    function updatePlaybackPosIndicator()
    {
        if(queuePage.currTrackDuration!==0)
            progressIndicator.indicator2Pos = (currPlaybackPos*progressIndicator.width) / (queuePage.currTrackDuration*1000);
    }

    function updateBufferingPosIndicator()
    {
        if(queuePage.currTrackDuration!==0) {
            if(currBufferingPos!==-1) {
                progressIndicator.indicator1Pos = (currBufferingPos*progressIndicator.width) / (queuePage.currTrackDuration*1000);
            }
            else { //current track buffered completely
                if(progressIndicator.indicator1Pos!=progressIndicator.width) {
                    progressIndicator.indicator1Pos = progressIndicator.width;
                }

            }
        }
    }


    /* toolbar element for the page */
    tools: commonToolBarLayout

    /*page header element */
    PageHeader
    {
        id: queuePageHeader
        displayComboIndicator: false
        text: qsTr("Now Playing")
        anchors.top: parent.top
        anchors.left: parent.left
    }

    /* ListView containing all currently queued tracks */
    ListView
    {
        id: playQueueView
        height: UiConstants.QUEUE_LISTVIEW_NORMAL_HEIGHT
        width: parent.width
        anchors.top: queuePageHeader.bottom
        anchors.topMargin: 10
        anchors.left: parent.left
        anchors.leftMargin: 5
        anchors.right: parent.right
        snapMode: ListView.SnapToItem
        model:  qspotCore.queueModel
        clip: true
        interactive: true
        delegate:   TrackInfoDelegate
        {
            id: queuedTrackItemDelegate
            highlightActiveTrack: true

            onClicked: {
                    setActiveTrack(index,false); //activate / play selected track
                }
           }
      }

    ScrollDecorator {
      flickableItem: playQueueView
    }

/* Main player elements (container for player controls / progress indicator / album cover */

/*Element holding the coverart for album associated with the currently playing track */
Image
{
    id: albumCover
    anchors.left: parent.left
    anchors.leftMargin: 5
    anchors.top: playQueueView.bottom
    anchors.topMargin: 10
    width: UiConstants.ALBUMCOVER_NORMAL_WIDTH
    height: UiConstants.ALBUMCOVER_NORMAL_HEIGHT
    smooth: true

    MouseArea
    {
        anchors.fill: parent

        onClicked: {
            centerViewAtCurrTrack(); //bring current playing track item into view
        }

        onDoubleClicked: {
            if(queuePage.currState===UiConstants.QUEUEPAGE_STATE_NORMAL)
                queuePage.currState = UiConstants.QUEUEPAGE_STATE_MINIMISED; //switch to minimised player state
            else
                queuePage.currState = UiConstants.QUEUEPAGE_STATE_NORMAL; //switch to normal player state
            updateBufferingPosIndicator();
            updatePlaybackPosIndicator();
        }
    }
}

Text
{
    id: trackNameLabel
    horizontalAlignment: Text.AlignHCenter
    font.family: UiConstants.DEFAULT_FONT_FAMILY
    font.pointSize: UiConstants.DEFAULT_TEXT_POINTSIZE
    anchors.left: albumCover.right
    anchors.top: albumCover.top
    anchors.leftMargin: 5
    wrapMode: Text.WordWrap
    color: "white"
}

Text
{
    id: artistNameLabel
    horizontalAlignment: Text.AlignHCenter
    font.family: UiConstants.DEFAULT_FONT_FAMILY
    font.pointSize: UiConstants.DEFAULT_SUBTEXT_POINTSIZE
    anchors.topMargin: 5
    anchors.top: trackNameLabel.bottom
    anchors.left: trackNameLabel.left
    width: 220
    elide: Text.ElideRight
    color: "white"
}

/* custom progress indicator element (shows current playback / buffering position) */
CustomProgressIndicator
{
    id: progressIndicator
    height: UiConstants.PROGRESS_INDICATOR_HEIGHT
    anchors.leftMargin: 5
    anchors.right: parent.right
    anchors.rightMargin: 5
    anchors.topMargin: 5
}

/* defines possible page states (i.e., the Now Playing view has a normal (maximised) and a minimised state) */
states: [
            State {
                name: "MINIMISED"
                when: (currState === UiConstants.QUEUEPAGE_STATE_MINIMISED)
                PropertyChanges {  target: playQueueView; height: UiConstants.QUEUE_LISTVIEW_MINIMISED_HEIGHT}
                PropertyChanges { target: albumCover; width: UiConstants.ALBUMCOVER_MINIMISED_WIDTH; height: UiConstants.ALBUMCOVER_MINIMISED_HEIGHT}
                PropertyChanges { target: progressIndicator;  anchors.bottomMargin: 5}
                PropertyChanges { target: trackNameLabel; width: UiConstants.TRACKNAME_MINIMISED_WIDTH}
                PropertyChanges {target: artistNameLabel; visible: false }
                AnchorChanges { target: progressIndicator; anchors.left: albumCover.right; anchors.verticalCenter: albumCover.verticalCenter;  anchors.top: undefined}
                AnchorChanges { target: trackNameLabel;  anchors.top: undefined}
                AnchorChanges { target: artistNameLabel;  anchors.top: undefined;  anchors.left: undefined }
            },

            State {
                name: "NORMAL"
                when: (currState === UiConstants.QUEUEPAGE_STATE_NORMAL)
                PropertyChanges {  target: playQueueView; height: UiConstants.QUEUE_LISTVIEW_NORMAL_HEIGHT}
                PropertyChanges { target: albumCover; width: UiConstants.ALBUMCOVER_NORMAL_WIDTH; height: UiConstants.ALBUMCOVER_NORMAL_HEIGHT}
                PropertyChanges { target: progressIndicator;  anchors.bottomMargin: 0}
                PropertyChanges { target: trackNameLabel; width: UiConstants.TRACKNAME_NORMAL_WIDTH}
                PropertyChanges {target: artistNameLabel; visible: true }
                AnchorChanges { target: progressIndicator; anchors.left: parent.left; anchors.verticalCenter: undefined;  anchors.top: albumCover.bottom}
                AnchorChanges { target: trackNameLabel;   anchors.top: albumCover.top}
                AnchorChanges { target: artistNameLabel;  anchors.top: trackNameLabel.bottom;  anchors.left: trackNameLabel.left}
            }
        ]

    //signal / slot connections
    Connections
    {
        target: qspotCore

        onImageLoaded: { //image has been successfully loaded
             albumCover.source = strPath + "/" + strImageID + ".jpg";
        }

        onCurrentTrackAssigned: { //current track has been set / activated

            if(pos!==-1)
            {
                setCurrentTrackDetails(durationSecs, durationFormatted, trackName, artistName);
            }
            else
            {
                setCurrentTrackDetails(0, "", "", "");
                albumCover.source = "";
            }

            if(pos!==playQueueView.currentIndex)
            {
                var defaultDuration = playQueueView.highlightMoveDuration;
                playQueueView.highlightMoveDuration = 1;
                playQueueView.currentIndex = pos; //update playqueue position
                playQueueView.highlightMoveDuration = defaultDuration;
            }
        }

        onPlaybackPositionUpdated: { //playback position updated (new position in slot parameter 'pos'; given in msecs)
            //calculate and set the current playback position in UI progress indicator
                currPlaybackPos = pos;
                updatePlaybackPosIndicator();
                progressIndicator.indicatorLabelLeft = posFormatted; //formatted position value
        }

        onBufferingPositionUpdated: { //buffering position updated (new position in slot parameter 'pos'; given in msecs)
             //calculate and set the current buffering position in UI progress indicator
                currBufferingPos = pos;
                updateBufferingPosIndicator();
        }

        onPlaybackStateChanged: { //playback state (paused / resumed) changed notification from backend
            if(isPlaying!==state)
                isPlaying = state;
          }
    }
}
